function [rings, local_ring] = pc_find_vertex_ring(v,k,c1,c2)
%% Search for k nearest neighbors (kNN)
numofv = size(v,1);

rings = cell(numofv,1);
local_ring = cell(numofv,1); %one-ring of the vertices

if size(v,2)==2
    v = [v, zeros(numofv,1)];
end

knnInd = knnsearch(v, v, 'K', k);

v_local_1 = reshape(v(knnInd,1), [numofv,k]);
v_local_2 = reshape(v(knnInd,2), [numofv,k]);
v_local_3 = reshape(v(knnInd,3), [numofv,k]);
X = zeros(numofv,k);
Y = zeros(numofv,k);
Z = zeros(numofv,k);
angle_sum = zeros(numofv,1);
Local_coord = cell(numofv,1);

for j=1:numofv
 %% Use PCA method to find the tangent plane and local coordinates w.r.t 
 % principal directions found by PCA.
  knn = zeros(k,3);
  % center to v(j,:)
  knn(:,1) = v_local_1(j,:)' - v(j,1);
  knn(:,2) = v_local_2(j,:)' - v(j,2);
  knn(:,3) = v_local_3(j,:)' - v(j,3);
  
  PCA = pca(knn,'Rows','complete');
  
  knn_local = knn * PCA;
  proj_k = knn - knn_local(:,3) * PCA(:,3)';
  knn_local(:,1:2) = proj_k * PCA(:,1:2);
  
  x_ = knn_local(:,1);
  y_ = knn_local(:,2);
  X(j,:) = x_';
  Y(j,:) = y_';
  Z(j,:) = knn_local(:,3)';
   
  KNNindex_j = knnInd(j,:);
  
  TRI = delaunay(x_,y_);
  
  [local_mesh_ind, ~] = find(KNNindex_j(TRI)==j);
  rings{j}=KNNindex_j(TRI(local_mesh_ind,:));
  Local_coord{j} = [x_,y_,KNNindex_j'];
  %% Find the neighbors in triangle form and want to delete the reference index
  local_tri = TRI(local_mesh_ind,:);
  local_v = [x_,y_];
  [row, col] = find(local_tri==1);
  indx = (1:length(row))';
  tmp = reshape(local_tri',3*size(local_tri,1),1);
  
  %% Va_j, Vb_j and V_j form a triangle
  tmp(3*indx-3+col)=[];
  local_ = reshape(tmp,2,size(tmp,1)/2)'; %local neighbors
  col1 = local_(:,1);
  col2 = local_(:,2);
  Va_j = local_v(col1,:);
  Vb_j = local_v(col2,:);
  
  %% Delete the triangles for angles > c2 and angles < c1
  Vc_j = Va_j-Vb_j;
  a_j = sqrt(Va_j(:,1).^2+Va_j(:,2).^2);
  b_j = sqrt(Vb_j(:,1).^2+Vb_j(:,2).^2);
  c_j = sqrt(Vc_j(:,1).^2+Vc_j(:,2).^2);
  angles = acosd((a_j.^2+b_j.^2-c_j.^2)./(2.*a_j.*b_j));
  transpose_ring = rings{j}';
  i_2 = reshape(transpose_ring(rings{j}'~=j), 2, size(rings{j},1))';
  v_aj = repmat(v(j,:),length(i_2),1) - v(i_2(:,1),:);
  v_bj = repmat(v(j,:),length(i_2),1) - v(i_2(:,2),:);
  v_cj = v(i_2(:,1),:) - v(i_2(:,2),:);
  a_j = sqrt(v_aj(:,1).^2+v_aj(:,2).^2);
  b_j = sqrt(v_bj(:,1).^2+v_bj(:,2).^2);
  c_j = sqrt(v_cj(:,1).^2+v_cj(:,2).^2);
  angles_ = acosd((a_j.^2+b_j.^2-c_j.^2)./(2.*a_j.*b_j));
  delete_indx1 = angles>c2;
  delete_indx2 = angles<c1;
  delete_indx3 = angles_>= 179.95;
  delete_indx4 = angles_<= 0.05;
  delete_indx = delete_indx1 + delete_indx2 + delete_indx3 + delete_indx4;
  delete_indx_int = delete_indx3 + delete_indx4;
  leng_delete_indx = length(delete_indx);
  delete_indx(delete_indx>1)=1;
  
  if length(angles)-ones(1,leng_delete_indx)*delete_indx~=0
      delete_indx = find(delete_indx>0);
      local_tri(delete_indx,:)=[];
      ring_j = rings{j};
      delete_indx_int = delete_indx_int>0;
      ring_j(delete_indx_int, :) = [];
      rings{j} = ring_j;

      angles(delete_indx)=[];
      angle_sum(j) = ones(1,size(local_tri,1))*angles;
  end
  
  %% Put one ring in KNNindex
  Transfer_index = [(1:k)',KNNindex_j'];
  return_matrix = zeros(size(local_tri,1),3);
  for ll = 1:size(local_tri,1)
      for ii = 1:3
          t = local_tri(ll,ii);
          return_matrix(ll,ii)=Transfer_index(t,2);
      end
  end
  local_ring{j} = return_matrix;
end

end