% Compute a free-boundary conformal parameterization for point clouds with 
% disk topology using the method in [1].
%
% Usage:
% map = pc_free_boundary_conformal_map(v,bdy_index,k,c1,c2)
% 
% Input:
% v: nv x 3 vertex coordinates of a point cloud with disk topology
% bdy_index: nb x 1 vertex indices of the boundary, with anti-clockwise orientation
% k: kNN parameter
% c1, c2: prescribed lower and upper bound for the boundary angles (in degrees)
%
% Output:
% map: nv x 2 vertex coordinates of the free boundary conformal parameterization
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, Y. Liu, and L. M. Lui,
%     "Free-boundary conformal parameterization of point clouds."
%     Journal of Scientific Computing, 90(1), 14, 2022.
%
% Copyright (c) 2020-2023, Gary P. T. Choi, Yechen Liu, Lok Ming Lui

addpath(genpath('code'));
addpath('data');

%% cloth
load('cloth.mat');

map = pc_free_boundary_conformal_map(v,bdy_index,k,c1,c2);

figure;
scatter3(v(:,1),v(:,2),v(:,3),'r','.');
axis equal tight off
view([-265 10]);

figure;
scatter(map(:,1),map(:,2),'r','.'); 
axis equal tight off

%% Julius
load('julius.mat');

map = pc_free_boundary_conformal_map(v,bdy_index,k,c1,c2);

figure;
scatter3(v(:,1),v(:,2),v(:,3),'r','.');
axis equal tight off
view([-170 -75]);

figure;
scatter(map(:,1),map(:,2),'r','.'); 
axis equal tight off
view([-90 90]);

%% spiral
load('spiral.mat');

map = pc_free_boundary_conformal_map(v,bdy_index,k,c1,c2);

figure;
scatter3(v(:,1),v(:,2),v(:,3),'r','.');
axis equal tight off
view([90 -80]);

figure;
scatter(map(:,1),map(:,2),'r','.'); 
axis equal tight off
